@@ -20,7 +20,6 @@
 
 # Import from the Standard Library
 from datetime import datetime
-from mimetypes import guess_all_extensions
 
 # Import from itools
 from itools.datatypes import String
@@ -29,10 +28,11 @@ from itools.handlers import File as FileHandler, Image as ImageHandler
 from itools.handlers import TARFile, ZIPFile, GzipFile, Bzip2File
 from itools.odf import SXWFile, SXCFile, SXIFile, ODTFile, ODSFile, ODPFile
 from itools.pdf import PDFFile
+from itools.utils import guess_all_extensions
 from itools.vfs import FileName
 from itools.web import STLView
-from itools.xml import MSWord as MSWordFile, MSExcel as MSExcelFile
 from itools.xml import MSPowerPoint as MSPowerPointFile, RTF as RTFFile
+from itools.xml import MSWord as MSWordFile, MSExcel as MSExcelFile
 
 # Import from ikaaro
 from registry import register_resource_class
