@@ -74,11 +74,6 @@ class Folder(DBResource):
     #######################################################################
     # API
     #######################################################################
-    def _has_resource(self, name):
-        folder = self.handler
-        return folder.has_handler('%s.metadata' % name)
-
-
     def _get_names(self):
         folder = self.handler
         return [ x[:-9] for x in folder.get_handler_names()
@@ -101,11 +96,18 @@ class Folder(DBResource):
         return cls(metadata)
 
 
-    def update_links(self, new_name):
+    def update_links(self, new_name, base_path):
         """The resource must update its links to itself and to its content
         """
+        # Check referencial-integrity
+        catalog = get_context().server.catalog
+        # The catalog is not available when updating (icms-update.py)
+        # FIXME We do not guarantee referencial-integrity when updating
+        if catalog is None:
+            return
+
         old_base_path = self.get_abspath()
-        new_base_path = old_base_path.resolve(new_name)
+        new_base_path = base_path.resolve2(new_name)
         for resource in self.traverse_resources():
             # Old and new paths
             old_path = resource.get_abspath()
@@ -114,27 +116,30 @@ class Folder(DBResource):
 
             # Get all the resources that have a link to this resource
             query = PhraseQuery('links', str(old_path))
-            results = self.get_root().search(query).get_documents()
+            results = catalog.search(query).get_documents()
             for result in results:
                 resource = self.get_resource(result.abspath)
                 resource.change_link(old_path, new_path)
 
 
     def del_resource(self, name):
+        context = get_context()
         resource = self.get_resource(name)
 
         # Check referencial-integrity
-        # FIXME Check sub-resources too
-        path = str(resource.get_abspath())
-        root = self.get_root()
-        results = root.search(links=path)
-        n = results.get_n_documents()
-        if n:
-            message = 'cannot delete, resource "%s" is referenced' % path
-            raise ConsistencyError, message
+        catalog = context.server.catalog
+        # The catalog is not available when updating (icms-update.py)
+        # FIXME We do not guarantee referencial-integrity when updating
+        if catalog is not None:
+            # FIXME Check sub-resources too
+            path = str(resource.get_abspath())
+            results = catalog.search(links=path)
+            if results.get_n_documents() > 0:
+                message = 'cannot delete, resource "%s" is referenced' % path
+                raise ConsistencyError, message
 
         # Events, remove
-        get_context().server.remove_resource(resource)
+        context.server.remove_resource(resource)
         # Remove
         folder = self.handler
         folder.del_handler('%s.metadata' % name)
@@ -143,9 +148,7 @@ class Folder(DBResource):
                 folder.del_handler(handler.uri)
 
 
-    def copy_resource(self, source, target):
-        context = get_context()
-
+    def _resolve_source_target(self, source, target):
         # Find out the source and target absolute URIs
         folder = self.handler
         if source[0] == '/':
@@ -156,14 +159,39 @@ class Folder(DBResource):
             target_uri = self.get_root().handler.uri.resolve2(target[1:])
         else:
             target_uri = folder.uri.resolve2(target)
-        old_name = source_uri.path[-1]
+
+        # Load the handlers so they are of the right class, for resources
+        # like that define explicitly the handler class.  This fixes for
+        # instance copy&cut&paste of a tracker in a just started server.
+        # TODO this is a work-around, there should be another way to define
+        # explicitly the handler class.
+        resource = self.get_resource(source)
+        if isinstance(resource, Folder):
+            for resource in resource.traverse_resources():
+                for handler in resource.get_handlers():
+                    pass
+        else:
+            for handler in resource.get_handlers():
+                pass
+
+        return source_uri, target_uri
+
+
+    def copy_resource(self, source, target):
+        server = get_context().server
+
+        # Find out the source and target absolute URIs
+        source_uri, target_uri = self._resolve_source_target(source, target)
         new_name = target_uri.path[-1]
 
+        # Get the source resource
+        resource = self.get_resource(source)
+
         # Copy the metadata
+        folder = self.handler
         folder.copy_handler('%s.metadata' % source_uri,
                             '%s.metadata' % target_uri)
         # Copy the content
-        resource = self.get_resource(source)
         for old_name, new_name in resource.rename_handlers(new_name):
             if old_name is None:
                 continue
@@ -174,32 +202,26 @@ class Folder(DBResource):
 
         # Events, add
         resource = self.get_resource(target)
-        context.server.add_resource(resource)
+        server.add_resource(resource)
 
 
     def move_resource(self, source, target):
-        context = get_context()
-        # Events, remove
-        resource = self.get_resource(source)
-        context.server.remove_resource(resource)
+        server = get_context().server
 
         # Find out the source and target absolute URIs
-        folder = self.handler
-        if source[0] == '/':
-            source_uri = self.get_root().handler.uri.resolve2(source[1:])
-        else:
-            source_uri = folder.uri.resolve2(source)
-        if target[0] == '/':
-            target_uri = self.get_root().handler.uri.resolve2(target[1:])
-        else:
-            target_uri = folder.uri.resolve2(target)
-        old_name = source_uri.path[-1]
+        source_uri, target_uri = self._resolve_source_target(source, target)
         new_name = target_uri.path[-1]
 
-        # The resource must update its links
-        resource.update_links(target)
+        # Get the source resource
+        resource = self.get_resource(source)
+
+        # Update the links to the resources that are to be moved
+        resource.update_links(target, self.get_abspath())
+        # Events, remove
+        server.remove_resource(resource)
 
         # Move the metadata
+        folder = self.handler
         folder.move_handler('%s.metadata' % source_uri,
                             '%s.metadata' % target_uri)
         # Move the content
@@ -213,7 +235,7 @@ class Folder(DBResource):
 
         # Events, add
         resource = self.get_resource(target)
-        context.server.add_resource(resource)
+        server.add_resource(resource)
 
 
     def traverse_resources(self):
