@@ -173,6 +173,8 @@ class Folder_Rename(STLForm):
         # Clean the copy cookie if needed
         cut, cp_paths = context.get_cookie('ikaaro_cp', type=CopyCookie)
 
+        renamed = []
+        referenced = []
         # Process input data
         abspath = resource.get_abspath()
         for i, path in enumerate(paths):
@@ -200,9 +202,19 @@ class Folder_Rename(STLForm):
                 context.del_cookie('ikaaro_cp')
                 cp_paths = []
             # Rename
-            container.move_resource(old_name, new_name)
+            try:
+                container.move_resource(old_name, new_name)
+            except ConsistencyError:
+                referenced.append(old_name)
+                continue
+            else:
+                renamed.append(old_name)
 
-        message = messages.MSG_RENAMED
+        if referenced and not renamed:
+            resources = ', '.join(referenced)
+            message = messages.MSG_RESOURCES_REFERENCED(resources=resources)
+        else:
+            message = messages.MSG_RENAMED
         return context.come_back(message, goto=';browse_content')
 
 
@@ -399,28 +411,22 @@ class Folder_BrowseContent(SearchForm):
             else:
                 not_removed.append(name)
 
+        message = []
         if removed:
             resources = ', '.join(removed)
-            message = messages.MSG_RESOURCES_REMOVED(resources=resources)
-            context.message = message
+            msg = messages.MSG_RESOURCES_REMOVED(resources=resources)
+            message.append(msg)
         if referenced:
             resources = ', '.join(referenced)
-            message = messages.MSG_RESOURCES_REFERENCED(resources=resources)
-            if context.message is None:
-                context.message = message
-            else:
-                # Merge messages
-                context.message = [context.message, message]
+            msg = messages.MSG_RESOURCES_REFERENCED(resources=resources)
+            message.append(msg)
         if not_removed:
             resources = ', '.join(not_removed)
-            message = messages.MSG_RESOURCES_NOT_REMOVED(resources=resources)
-            if context.message is None:
-                context.message = message
-            else:
-                # Merge messages
-                context.message = [context.message, message]
+            msg = messages.MSG_RESOURCES_NOT_REMOVED(resources=resources)
+            message.append(msg)
         if not removed and not referenced and not not_removed:
-            context.message = messages.MSG_NONE_REMOVED
+            message.append(messages.MSG_NONE_REMOVED)
+        context.message = message
 
 
     def action_rename(self, resource, context, form):
