@@ -164,13 +164,28 @@ class ReadOnlyWidget(Widget):
 
 
     def get_namespace(self, datatype, value):
+        displayed = getattr(self, 'displayed', None)
+
         if issubclass(datatype, Enumerate) and isinstance(value, list):
-            selected = [x for x in value if x['selected'] is True]
-            value = selected[0]['value'] if selected else datatype.default
+            for option in value:
+                if not option['selected']:
+                    continue
+                value = option['name']
+                if displayed is None:
+                    displayed = option['value']
+                break
+            else:
+                value = datatype.default
+                if displayed is None:
+                    displayed = datatype.get_value(value)
+
+        if displayed is None:
+            displayed = value
+
         return {
             'name': self.name,
             'value': value,
-            'displayed': getattr(self, 'displayed', value)}
+            'displayed': displayed}
 
 
 
@@ -284,11 +299,13 @@ class SelectWidget(Widget):
 class SelectRadio(Widget):
 
     template = list(XMLParser("""
-        <input type="radio" name="${name}" value="" checked="checked"
-          stl:if="none_selected"/>
-        <input type="radio" name="${name}" value=""
-          stl:if="not none_selected"/>
-        <br/>
+        <stl:block stl:if="has_empty_option">
+          <input type="radio" name="${name}" value="" checked="checked"
+            stl:if="none_selected"/>
+          <input type="radio" name="${name}" value=""
+            stl:if="not none_selected"/>
+          <br/>
+        </stl:block>
         <stl:block stl:repeat="option options">
           <input type="radio" id="${name}_${option/name}" name="${name}"
             value="${option/name}" checked="checked"
@@ -325,16 +342,23 @@ class SelectRadio(Widget):
         else:
             options = value
 
+        has_empty_option = getattr(self, 'has_empty_option', True)
         for option in options:
             if option['selected'] is True:
                 none_selected = False
                 break
+        else:
+            # Select first item if no empty option
+            if not has_empty_option and options:
+                options[0]['selected'] = True
         return {
             'name': self.name,
+            'has_empty_option': has_empty_option,
             'none_selected': none_selected,
             'options': options}
 
 
+
 class DateWidget(Widget):
 
     template = list(XMLParser("""
@@ -465,6 +489,8 @@ class RTEWidget(Widget):
     toolbar3 = None
     resizing = True
     plugins = 'safari,table,media'
+    # Extending the existing rule set.
+    extended_valid_elements = None
     # css
     advanced_styles = None
     table_styles = None
@@ -504,20 +530,21 @@ class RTEWidget(Widget):
         resizing = 'true' if self.resizing else 'false'
 
         css_names = self.get_rte_css()
-        return {'form_name': self.name,
-                'source': value,
-                'scripts': self.rte_scripts,
+        return {'advanced_styles': self.advanced_styles,
                 'css': ','.join(css_names),
-                'language': current_language,
-                'width': self.width,
+                'extended_valid_elements': self.extended_valid_elements,
+                'form_name': self.name,
                 'height': self.height,
+                'language': current_language,
+                'plugins': self.plugins,
+                'resizing': resizing,
+                'scripts': self.rte_scripts,
+                'source': value,
+                'table_styles': self.table_styles,
                 'toolbar1': self.toolbar1,
                 'toolbar2': self.toolbar2,
                 'toolbar3': self.toolbar3,
-                'resizing': resizing,
-                'plugins': self.plugins,
-                'advanced_styles': self.advanced_styles,
-                'table_styles': self.table_styles}
+                'width': self.width}
 
 
 ###########################################################################
