@@ -14,28 +14,44 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+# Import from standard Library
+from copy import deepcopy
+
 # Import from itools
-from itools.csv import Property
 from itools.datatypes import String, Enumerate, Unicode, Integer
 from itools.gettext import MSG
-from itools.uri import Path
+from itools.handlers import checkid
+from itools.uri import Path, get_reference
+from itools.web import get_context
 from itools.xml import XMLParser
 
 # Import from ikaaro
-from ikaaro.buttons import Button
-from ikaaro.file import File
-from ikaaro.folder import Folder
-from ikaaro.forms import TextWidget, SelectWidget, ReadOnlyWidget
-from ikaaro.forms import stl_namespaces, PathSelectorWidget
 from ikaaro import messages
+from ikaaro.buttons import Button
+from ikaaro.exceptions import ConsistencyError
+from ikaaro.folder import Folder
+from ikaaro.forms import PathSelectorWidget
+from ikaaro.forms import TextWidget, SelectWidget, ReadOnlyWidget
 from ikaaro.registry import register_resource_class
-from ikaaro.resource_views import Breadcrumb, DBResource_AddLink
 from ikaaro.table import OrderedTableFile, OrderedTable
 from ikaaro.table_views import OrderedTable_View
 from ikaaro.workflow import WorkflowAware
 
 
 
+def get_reference_and_path(path):
+    """Return the reference associated to the path and the path
+    without query/fragment.
+    """
+    ref = get_reference(path)
+    return ref, str(ref.path)
+
+
+class NotAllowedError(Exception):
+    pass
+
+
+
 class Target(Enumerate):
 
     options = [{'name': '_blank', 'value': MSG(u'New page')},
@@ -97,14 +113,22 @@ class Menu_View(OrderedTable_View):
         value = get_value(item, column)
         if column == 'title':
             path = get_value(item, 'path')
-            if path.startswith(('http://', 'https://')) or path.count(';'):
-                # External link
+            # Get the reference and path
+            ref, path = get_reference_and_path(path)
+            if ref.scheme or path.count(';'):
+                # External link or method
                 return value, path
+            if ref.path.is_absolute():
+                site_root = context.resource.get_site_root()
+                path = site_root.get_abspath().resolve2('.%s' % path)
             if resource.has_resource(path) is False:
                 # Broken link
                 return value
             resource_item = resource.get_resource(path)
-            return value, context.get_link(resource_item)
+            # Build the new reference with the right path
+            ref2 = deepcopy(ref)
+            ref2.path = context.get_link(resource_item)
+            return value, str(ref2)
         elif column == 'child':
             child = None
             parent = resource.parent
@@ -115,9 +139,14 @@ class Menu_View(OrderedTable_View):
             return child
         elif column == 'workflow_state':
             path = get_value(item, 'path')
-            if path.startswith(('http://', 'https://')) or path.count(';'):
-                # External link
+            # Get the reference and path
+            ref, path = get_reference_and_path(path)
+            if ref.scheme or path.count(';'):
+                # External link or method
                 return None
+            if ref.path.is_absolute():
+                site_root = context.resource.get_site_root()
+                path = site_root.get_abspath().resolve2('.%s' % path)
             if resource.has_resource(path) is False:
                 # Broken link
                 title = MSG(u'The resource does not exist anymore.')
@@ -134,11 +163,13 @@ class Menu_View(OrderedTable_View):
             statename = item_resource.get_statename()
             state = item_resource.get_state()
             msg = state['title'].gettext().encode('utf-8')
-            path = context.get_link(item_resource)
+            # Build the new reference with the right path
+            ref2 = deepcopy(ref)
+            ref2.path = context.get_link(item_resource)
             # TODO Include the template in the base table
             state = ('<a href="%s/;edit_state" class="workflow">'
                      '<strong class="wf_%s">%s</strong>'
-                     '</a>') % (path, statename, msg)
+                     '</a>') % (str(ref2), statename, msg)
             return XMLParser(state)
 
         return OrderedTable_View.get_item_value(self, resource, context, item,
@@ -154,53 +185,38 @@ class Menu_View(OrderedTable_View):
     #######################################################################
     def action_remove(self, resource, context, form):
         ids = form['ids']
+        removed = []
+        not_removed = []
+        referenced = []
         for id in ids:
-            resource.before_remove_record(id)
-            resource.handler.del_record(id)
+            try:
+                resource.del_record(id)
+            except ConsistencyError, e:
+                referenced.append(str(id))
+            except NotAllowedError:
+                not_removed.append(str(id))
+            else:
+                removed.append(str(id))
 
-        context.message = MSG(u'Record deleted.')
+        message = []
+        if removed:
+            resources = ', '.join(removed)
+            msg = messages.MSG_RESOURCES_REMOVED(resources=resources)
+            message.append(msg)
+        if referenced:
+            resources = ', '.join(referenced)
+            msg = messages.MSG_RESOURCES_REFERENCED(resources=resources)
+            message.append(msg)
+        if not_removed:
+            resources = ', '.join(not_removed)
+            msg = messages.MSG_RESOURCES_NOT_REMOVED(resources=resources)
+            message.append(msg)
+        if not removed and not referenced and not not_removed:
+            message.append(messages.MSG_NONE_REMOVED)
+        context.message = message
 
-
-    def action_order_up(self, resource, context, form):
-        ids = form['ids']
-        if not ids:
-            context.message = MSG(u'Please select the objects to order up.')
-            return
-
-        resource.handler.order_up(ids)
-        context.message = MSG(u'Objects ordered up.')
-
-
-    def action_order_down(self, resource, context, form):
-        ids = form['ids']
-        if not ids:
-            context.message = MSG(u'Please select the objects to order down.')
-            return
-
-        resource.handler.order_down(ids)
-        context.message = MSG(u'Objects ordered down.')
-
-
-    def action_order_top(self, resource, context, form):
-        ids = form['ids']
-        if not ids:
-            message = MSG(u'Please select the objects to order on top.')
-            context.message = message
-            return
-
-        resource.handler.order_top(ids)
-        context.message = MSG(u'Objects ordered on top.')
-
-
-    def action_order_bottom(self, resource, context, form):
-        ids = form['ids']
-        if not ids:
-            message = MSG(u'Please select the objects to order on bottom.')
-            context.message = message
-            return
-
-        resource.handler.order_bottom(ids)
-        context.message = MSG(u'Objects ordered on bottom.')
+        # Reindex the resource
+        context.server.change_resource(resource)
 
 
     def action_add_child(self, resource, context, form):
@@ -216,18 +232,17 @@ class Menu_View(OrderedTable_View):
 
             names = parent.get_names()
             index = len(names) / 2
-            base = 'menu_'
-            name = '%s%03d' % (base, index)
+            base = 'menu '
+            name = checkid('%s%03d' % (base, index))
 
             while name in names:
                 index = index + 1
-                name = '%s%03d' % (base, index)
+                name = checkid('%s%03d' % (base, index))
 
             object = Menu.make_resource(Menu, parent, name)
 
             # update the parent record
-            handler.update_record(parent_id,
-                                           **{'child': name})
+            handler.update_record(parent_id, **{'child': name})
 
         context.message = messages.MSG_NEW_RESOURCE
 
@@ -243,62 +258,135 @@ class Menu(OrderedTable):
     # Views
     view = Menu_View()
 
+
     form = [TextWidget('title', title=MSG(u'Title')),
             PathSelectorWidget('path', title=MSG(u'Path')),
             SelectWidget('target', title=MSG(u'Target')),
             ReadOnlyWidget('child')]
 
 
-    def before_remove_record(self, id):
+    def del_record(self, id):
         handler = self.handler
         record = handler.get_record(id)
-        child_path = handler.get_record_value(record, 'child')
-        container = self.parent
-        if child_path and container.has_resource(child_path):
-            child = container.get_resource(child_path)
-            for record_id in child.handler.get_record_ids():
-                child.before_remove_record(record_id)
-            container.del_resource(child_path)
+        record_schema = handler.record_schema
+
+        # Delete submenu
+        if 'child' in record_schema:
+            child_path = handler.get_record_value(record, 'child')
+            container = self.parent
+            user = get_context().user
+            if child_path and container.has_resource(child_path):
+                child = container.get_resource(child_path)
+                ac = child.get_access_control()
+                if ac.is_allowed_to_remove(user, child):
+                    if child.handler.get_n_records():
+                        raise NotAllowedError
+                    # Remove the child table
+                    # May raise a ConsistencyError
+                    container.del_resource(child_path)
+                else:
+                    raise NotAllowedError
+
+        # Delete the record
+        handler.del_record(id)
+
+
+    def _is_allowed_to_access(self, context, uri):
+        # Get the reference and path
+        ref, path = get_reference_and_path(uri)
+        if ref.scheme:
+            # External link
+            return True
+
+        user = context.user
+        site_root_abspath = context.resource.get_site_root().get_abspath()
+        if ref is None or path == '':
+            # Skip broken entry
+            return False
+        elif path.count(';'):
+            path, method = path.split(';')
+            if ref.path.is_absolute():
+                path = site_root_abspath.resolve2('.%s' % path)
+            if self.has_resource(path):
+                resource = self.get_resource(path)
+                ac = resource.get_access_control()
+                view = resource.get_view(method, ref.query)
+                # Check ACL
+                if view:
+                    return ac.is_access_allowed(user, resource, view)
+            # Miss
+            return False
+        else:
+            # Internal link
+            if ref.path.is_absolute():
+                path = site_root_abspath.resolve2('.%s' % path)
+            if self.has_resource(path) is False:
+                # Broken link
+                return False
+
+            # FIXME We should take into account the show_first_child
+            # parameter
+            resource = self.get_resource(path)
+            ac = resource.get_access_control()
+            return ac.is_allowed_to_view(user, resource)
 
 
     def get_menu_namespace_level(self, context, url, depth=2,
                                  use_first_child=False, flat=False):
         parent = self.parent
         handler = self.handler
-        site_root = context.site_root
-        user = context.user
-        here_abspath = context.resource.get_abspath()
+        here = context.resource
+        here_abspath = here.get_abspath()
+        site_root_abspath = here.get_site_root().get_abspath()
         items = []
         tabs = {}
         get_value = handler.get_record_value
         for record in handler.get_records_in_order():
             # Get the objects, check security
-            path = get_value(record, 'path')
+            uri = get_value(record, 'path')
+            if self._is_allowed_to_access(context, uri) is False:
+                continue
             title = get_value(record, 'title')
             target = get_value(record, 'target')
             # Subtabs
             subtabs = {}
-            if path.startswith(('http://', 'https://')) or path.count(';'):
-                # Special case for external link & method
-                items.append({'id': 'menu_', # FIXME
-                              'path': path,
+            # Get the reference and path
+            ref, path = get_reference_and_path(uri)
+            if ref.scheme or path.count(';'):
+                # Special case for external link and method
+                id = 'menu_'
+                css = None
+                if not ref.scheme:
+                    # FIXME We should check the ACL of the method
+                    # Get the resource associated to the method
+                    _path, _method = path.split(';')
+                    if ref.path.is_absolute():
+                        real_path = site_root_abspath.resolve2('.%s' % _path)
+                    else:
+                        real_path = _path
+                    resource = self.get_resource(real_path)
+                    if here_abspath == resource.get_abspath():
+                        if context.view_name == _method:
+                            css = 'in_path'
+                    id += _method
+                items.append({'id': id,
+                              'path': str(ref),
+                              'real_path': None,
                               'title': title,
-                              'description': None, # FIXME
+                              'description': None,
                               'in_path': False,
                               'active': False,
-                              'class': None,
+                              'class': css,
                               'target': target})
             else:
-                if self.has_resource(path) is False:
-                    # Broken link
-                    continue
+                # Internal link
+                if ref.path.is_absolute():
+                    path = site_root_abspath.resolve2('.%s' % path)
                 resource = self.get_resource(path)
                 name = resource.name
 
-                ac = resource.get_access_control()
-                if ac.is_allowed_to_view(user, resource) is False:
-                    continue
-
+                # use first child by default we use the resource itself
+                resource_path = path
                 if depth > 1:
                     # Check the child
                     child_path = get_value(record, 'child')
@@ -308,6 +396,13 @@ class Menu(OrderedTable):
                         get_menu_ns_lvl = child.get_menu_namespace_level
                         subtabs = get_menu_ns_lvl(context, url, depth - 1,
                                                   use_first_child, flat)
+                        # use first child
+                        if subtabs['items']:
+                            sub_path = subtabs['items'][0]['real_path']
+                            # if the first child is an external link => skip it
+                            if sub_path is not None:
+                                resource_path = sub_path
+
                 # Set active, in_path
                 active, in_path = False, name in url
                 if here_abspath == resource.get_abspath():
@@ -316,8 +411,13 @@ class Menu(OrderedTable):
                 # Set css class to 'active', 'in_path' or None
                 css = (active and 'in_path') or (in_path and 'in_path') or None
 
+                # Build the new reference with the right path
+                ref2 = deepcopy(ref)
+                resource = self.get_resource(path)
+                ref2.path = context.get_link(resource)
                 items.append({'id': 'menu_%s' % name,
-                              'path': context.get_link(resource),
+                              'path': str(ref2),
+                              'real_path': resource.get_abspath(),
                               'title': title,
                               'description': None, # FIXME
                               'in_path': css == 'in_path' or active,
@@ -329,17 +429,75 @@ class Menu(OrderedTable):
         return tabs
 
 
+    def get_first_level_uris(self, context):
+        """Return a list of the URI of all entries in the first level
+        """
+        handler = self.handler
+        site_root = context.resource.get_site_root()
+        site_root_abspath = site_root.get_abspath()
+        user = context.user
+        get_value = handler.get_record_value
+        uris = []
+        for record in handler.get_records_in_order():
+            # Get the objects, check security
+            uri = get_value(record, 'path')
+            # Get the reference and path
+            ref, path = get_reference_and_path(uri)
+            if ref.scheme:
+                # External link
+                uris.append(uri)
+                continue
+            if self._is_allowed_to_access(context, uri) is False:
+                continue
+            if path.count(';'):
+                path, method = path.split(';')
+                if ref.path.is_absolute():
+                    path = site_root_abspath.resolve2('.%s' % path)
+                resource = self.get_resource(path)
+            else:
+                path = site_root_abspath.resolve2('.%s' % path)
+                resource = self.get_resource(path)
+            # Build the new reference with the right path
+            ref2 = deepcopy(ref)
+            ref2.path = context.get_link(resource)
+            uris.append(ref2)
+
+        return uris
+
+
     def get_links(self):
         base = self.get_abspath()
+        # FIXME The context is not available when updating the catalog.
+        site_root_abspath = self.parent.parent.get_abspath()
         handler = self.handler
+        record_schema = handler.record_schema
         links = []
 
         for record in handler.get_records_in_order():
+            # Target resources
             path = handler.get_record_value(record, 'path')
-            if path.startswith(('http://', 'https://')) or path.count(';'):
+            ref, path = get_reference_and_path(path)
+            if ref.scheme:
                 continue
-            uri = base.resolve2(path)
+            if path.count(';'):
+                path, method = path.split(';')
+            if ref.path.is_absolute():
+                uri = site_root_abspath.resolve2('.%s' % path)
+            else:
+                uri = base.resolve2(path)
             links.append(str(uri))
+            # Submenu resources
+            if not 'child' in record_schema:
+                continue
+            path = handler.get_record_value(record, 'child')
+            if path:
+                container = self.parent
+                if container.has_resource(path):
+                    # take into account the submenu if it is not empty
+                    child = container.get_resource(path)
+                    if child.handler.get_n_records():
+                        uri = site_root_abspath.resolve(path)
+                        links.append(str(uri))
 
         return links
 
@@ -347,12 +505,21 @@ class Menu(OrderedTable):
     def change_link(self, old_path, new_path):
         handler = self.handler
         base = self.get_abspath()
+        # FIXME The context is not available when updating the catalog.
+        site_root_abspath = self.parent.parent.get_abspath()
 
         for record in handler.get_records_in_order():
             path = handler.get_record_value(record, 'path')
-            if path.startswith(('http://', 'https://')) or path.count(';'):
+            ref, path = get_reference_and_path(path)
+            if ref.scheme:
                 continue
-            uri = str(base.resolve2(path))
+            if path.count(';'):
+                path, method = path.split(';')
+            if ref.path.is_absolute():
+                uri = site_root_abspath.resolve2('.%s' % path)
+            else:
+                uri = base.resolve2(path)
+            uri = str(uri)
             if uri == old_path:
                 # Hit the old name
                 new_path2 = base.get_pathto(Path(new_path))
@@ -388,6 +555,11 @@ class MenuFolder(Folder):
                                                   use_first_child, flat)
 
 
+    def get_first_level_uris(self, context):
+        menu_root = self.get_resource('menu')
+        return menu_root.get_first_level_uris(context)
+
+
 
 def get_menu_namespace(context, depth=3, show_first_child=False, flat=True,
                        src=None):
