@@ -15,9 +15,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.datatypes import String
+from itools.datatypes import String, Tokens
 from itools.gettext import MSG
+from itools.handlers import merge_dicts
 from itools.web import INFO
+from itools.xapian import AndQuery, PhraseQuery, StartQuery
 from itools.xml import XMLParser
 
 # Import from ikaaro
@@ -40,7 +42,11 @@ class AddButton(Button):
 
 
 
-class ChildrenOrderedTable_Ordered(OrderedTable_View):
+###########################################################################
+# ResourcesOrderedTable
+###########################################################################
+
+class ResourcesOrderedTable_Ordered(OrderedTable_View):
 
     title = MSG('Ordered items')
 
@@ -55,11 +61,11 @@ class ChildrenOrderedTable_Ordered(OrderedTable_View):
 
     def get_item_value(self, resource, context, item, column):
         if column == 'title':
-            item = resource.parent.get_resource(item.name)
+            item = resource.get_resource_by_name(item.name, context)
             return item.get_title(), context.get_link(item)
         elif column == 'workflow_state':
             # The workflow state
-            item = resource.parent.get_resource(item.name)
+            item = resource.get_resource_by_name(item.name, context)
             if not isinstance(item, WorkflowAware):
                 return None
             statename = item.get_statename()
@@ -75,8 +81,7 @@ class ChildrenOrderedTable_Ordered(OrderedTable_View):
                                                 column)
 
 
-
-class ChildrenOrderedTable_Unordered(Folder_BrowseContent):
+class ResourcesOrderedTable_Unordered(Folder_BrowseContent):
 
     access = 'is_allowed_to_edit'
     title = MSG('Unordered items')
@@ -102,9 +107,18 @@ class ChildrenOrderedTable_Unordered(Folder_BrowseContent):
     def get_items(self, resource, context):
         exclude = [resource.name] + list(resource.get_ordered_names())
         orderable_classes = resource.get_orderable_classes() or ()
+        path = resource.get_root_order_path(context)
+
+        query = [StartQuery('abspath', str(path))]
+        for cl in orderable_classes:
+            query.append(PhraseQuery('format', cl.class_id))
+        query = AndQuery(*query)
+        root = context.root
+        results = root.search(query).get_documents()
         items = []
-        for item in resource.parent.search_resources(cls=orderable_classes):
+        for item in results:
             if item.name not in exclude:
+                item = root.get_resource(item.abspath)
                 items.append(item)
         return items
 
@@ -118,7 +132,6 @@ class ChildrenOrderedTable_Unordered(Folder_BrowseContent):
             return item.name
         if column == 'workflow_state':
             # The workflow state
-            item = resource.parent.get_resource(item.name)
             if not isinstance(item, WorkflowAware):
                 return None
             statename = item.get_statename()
@@ -137,7 +150,6 @@ class ChildrenOrderedTable_Unordered(Folder_BrowseContent):
 
 
     def action_add(self, resource, context, form):
-        parent = resource.parent
         handler = resource.handler
 
         order = resource.get_property('order') or []
@@ -155,14 +167,14 @@ class ChildrenOrderedTable_Unordered(Folder_BrowseContent):
 
 
 
-class ChildrenOrderedTable_View(CompositeForm):
+class ResourcesOrderedTable_View(CompositeForm):
 
     access = 'is_allowed_to_edit'
     title = MSG(u'View')
     template = '/ui/future/order_view.xml'
 
-    subviews = [ChildrenOrderedTable_Ordered(),
-                ChildrenOrderedTable_Unordered()]
+    subviews = [ResourcesOrderedTable_Ordered(),
+                ResourcesOrderedTable_Unordered()]
 
 
     def get_namespace(self, resource, context):
@@ -173,6 +185,34 @@ class ChildrenOrderedTable_View(CompositeForm):
         return {'views': views}
 
 
+###########################################################################
+# ChildrenOrderedTable
+###########################################################################
+
+class ChildrenOrderedTable_Ordered(ResourcesOrderedTable_Ordered):
+    pass
+
+
+
+class ChildrenOrderedTable_Unordered(ResourcesOrderedTable_Unordered):
+
+    def get_items(self, resource, context):
+        exclude = [resource.name] + list(resource.get_ordered_names())
+        orderable_classes = resource.get_orderable_classes() or ()
+        items = []
+        for item in resource.parent.search_resources(cls=orderable_classes):
+            if item.name not in exclude:
+                items.append(item)
+        return items
+
+
+
+class ChildrenOrderedTable_View(ResourcesOrderedTable_View):
+
+    subviews = [ChildrenOrderedTable_Ordered(),
+                ChildrenOrderedTable_Unordered()]
+
+
 
 class ChildrenOrderedTable_AddLink(DBResource_AddLink):
 
@@ -205,32 +245,87 @@ class ChildrenOrderedTable_AddLink(DBResource_AddLink):
         return namespace
 
 
+###########################################################################
+# Resources
+###########################################################################
 
-class ChildrenOrderedTableFile(OrderedTableFile):
+class ResourcesOrderedTableFile(OrderedTableFile):
 
+    record_schema = {'name': String(mandatory=True, unique=True, index='keyword')}
+
+
+
+class ChildrenOrderedTableFile(ResourcesOrderedTableFile):
+    # FIXME 0.60 -> to remove
     record_schema = {'name': String}
 
 
 
-class ChildrenOrderedTable(OrderedTable):
+class ResourcesOrderedTable(OrderedTable):
 
-    class_id = 'children-ordered-table'
-    class_title = MSG(u'Children Ordered Table')
-    class_handler = ChildrenOrderedTableFile
+    class_id = 'resources-ordered-table'
+    class_title = MSG(u'Resources Ordered Table')
+    class_handler = ResourcesOrderedTableFile
 
     orderable_classes = None
     form = [PathSelectorWidget('name', title=MSG(u'Path'))]
 
 
+    @classmethod
+    def get_metadata_schema(cls):
+        return merge_dicts(OrderedTable.get_metadata_schema(),
+                           order=Tokens(default=()))
+
+
     def get_orderable_classes(self):
         return self.orderable_classes
 
 
+    def get_root_order_path(self, context):
+        """ Every item will be into this path. """
+        return context.root.get_abspath()
+
+
     def get_ordered_names(self):
         for record in self.handler.get_records_in_order():
             yield record.name
 
 
+    def get_resource_by_name(self, name, context):
+        root_order_path = self.get_root_order_path(context)
+        folder = self.get_resource(root_order_path)
+        return folder.get_resource(name)
+
+
+    def get_links(self):
+        base = self.get_abspath()
+        handler = self.handler
+        links = []
+
+        for record in handler.get_records_in_order():
+            # Target resources
+            path = handler.get_record_value(record, 'name')
+            links.append(str(base.resolve(path)))
+
+        return links
+
+
+
+class ChildrenOrderedTable(ResourcesOrderedTable):
+
+    class_id = 'children-ordered-table'
+    class_title = MSG(u'Children Ordered Table')
+    class_handler = ChildrenOrderedTableFile
+
+
+    def get_root_order_path(self, context):
+        """ Every item will be into this path. """
+        return self.parent.get_abspath()
+
+
+    def get_resource_by_name(self, name, context):
+        return self.parent.get_resource(name)
+
     # Views
     add_link = ChildrenOrderedTable_AddLink()
     ordered = ChildrenOrderedTable_Ordered()
@@ -238,4 +333,5 @@ class ChildrenOrderedTable(OrderedTable):
     view = ChildrenOrderedTable_View()
 
 
+register_resource_class(ResourcesOrderedTable)
 register_resource_class(ChildrenOrderedTable)
