@@ -39,12 +39,15 @@ from itools.xml import XMLParser, START_ELEMENT, END_ELEMENT, TEXT
 from ikaaro.messages import MSG_CHANGES_SAVED
 from ikaaro.table_views import Table_View
 from ikaaro.views import CompositeForm
+from ikaaro.views import ContextMenu
 
 # Local import
 from datatypes import get_issue_fields, UsersList
 
 
-
+###########################################################################
+# Utilities
+###########################################################################
 url_expr = compile('([fh]t?tps?://[\w;/?:@&=+$,.#\-%]*)')
 class OurWrapper(TextWrapper):
 
@@ -82,8 +85,8 @@ def indent(text):
     # Wrap
     buffer = []
     text_wrapper = OurWrapper(95)
-    for line in text.splitlines(True):
-        line = text_wrapper.fill(line)
+    for line in text.splitlines():
+        line = text_wrapper.fill(line) + '\n'
         for segment in url_expr.split(line):
             url = text_wrapper.urls_map.get(segment)
             if url is None:
@@ -103,6 +106,26 @@ def indent(text):
 
 
 
+###########################################################################
+# Menu
+###########################################################################
+class IssueTrackerMenu(ContextMenu):
+
+    title = MSG(u'Tracker')
+
+    def get_items(self, resource, context):
+        items = [
+            {'title': MSG(u'Search for issues'),
+             'href': '%s/;search' % context.get_link(resource.parent)},
+            {'title': MSG(u'Add a new issue'),
+             'href': '%s/;add_issue' % context.get_link(resource.parent)}]
+        return items
+
+
+
+###########################################################################
+# Views
+###########################################################################
 class Issue_Edit(STLForm):
 
     access = 'is_allowed_to_edit'
